/*
 * 	Doku, siehe http://www.mikrocontroller.net/articles/AVR_FAT32
 * 	Neuste Version: http://www.mikrocontroller.net/svnbrowser/avr-fat32/
 * 	Autor: Daniel R.
 */


#ifndef CONFIG_H_
	#define CONFIG_H_

	//#####################################################################################
	// BOOL DEFINITIONEN

	#define TRUE 	1
	#define FALSE 	0

	//#####################################################################################
	// FLAGS

	// flags fuer ffopen (sind die rueckgabewerte von ffopen, auf diese wird geprueft..)
	#define MMC_FILE_NEW 			0	// datei existierte nicht, wurde gerade angelegt !
	#define MMC_FILE_EXISTS 		1	// datei existiert, es kann gelesen werden !
	#define MMC_FILE_NOT_EXISTS 		2	// datei existiert nicht, wird nur bei read-only zurueckgegeben, wenn die datei nicht gefunden wurde !


	//#####################################################################################
	// WICHTIGE SCHALTER

	// schalter die den funktionsumfang aendern bzw, den funktionsumfang einiger funktionen :)
	#define MMC_WRITE 			TRUE	// TRUE, dann mit write unterstuetzung, wenn FALSE dann read only !
	#define MMC_OVER_WRITE 			FALSE	// TRUE und MMC_WRITE TRUE, dann kann ffwrite dateien überschreiben, wenn FALSE dann nur normales schreiben. um an eine datei anzuhaengen ist MMC_OVER_WRITE TRUE nicht noetig!
	#define MMC_MULTI_BLOCK 		FALSE	// TRUE und MMC_OVER_WRITE FALSE, dann werden multiblock schreib/lese funktionen benutzt. ist schneller, wird aber möglicherweise nicht von allen karten unterstützt. wenn FALSE ist normale operation
	#define MMC_SDHC_SUPPORT		FALSE	// TRUE, dann mit sdhc unterstuetzung, wenn FALSE, dann nur mmc/sd karten. siehe : http://de.wikipedia.org/wiki/SD_Memory_Card#SDHC_.28SD_2.0.29
	#define MMC_ENDIANNESS_LITTLE		TRUE	// TRUE, dann ist der code auf littleendian ausgelegt. AVR ist littleendian. code ist auf littleendian optimiert!! siehe: http://de.wikipedia.org/wiki/Endianness
	#define MMC_LFN_SUPPORT			FALSE	// TRUE, dann mit unterstuetzung fuer lange dateinamen. kostet wenn read und write benutzt wird um die 800 bytes flash...
	#define MMC_RM_FILES_ONLY		TRUE	// TRUE ,MMC_WRITE TRUE und MMC_RM TRUE, dann wird die funktion ffrm so mit kompiliert, dass sie nur dateien loeschen kann. wenn FALSE und MMC_WRITE TRUE, dann kann die funktion dateien und ordner rekursiv loeschen !

	// schalter die explizit funktionen mit kompilieren oder nicht!
	#define MMC_TIME_STAMP 			FALSE 	// TRUE, dann werden die funktionen fat_getTime und fat_getFreeBytes mit kompiliert. siehe auch abschnitt: ZEIT FUNKTIONEN, weiter unten
	#define MMC_RM 				FALSE	// TRUE und MMC_WRITE TRUE, dann wird die funktion ffrm mit kompiliert.
	#define MMC_SEEK			TRUE	// TRUE,dann wird die funktion ffseek mit kompiliert. mit dieser funktion kann man in einer geoeffneten datei vor und zurueck spulen. nur in kombination mit MMC_OVER_WRITE TRUE kann in einer datei ueberschrieben werden.
	#define MMC_MKDIR			FALSE	// TRUE und MMC_WRITE TRUE, dann wird die funktion ffmkdir mit kompiliert. mit dieser funktion kann man ordner anlegen.
	#define MMC_GET_FREE_BYTES		FALSE	// TRUE, dann wird die funkton fat_getFreeBytes mit kompiliert. mit dieser funktion kann der freie platzt auf der karte ermittelt werden
	#define MMC_LS				FALSE	// TRUE, dann wird die funktion ffls mit kompiliert. mit dieser funkion kann man die dateien auf der karte anzeigen lassen
	#define MMC_CD				FALSE	// TRUE, dann werden die funktionen ffcd und ffcdLower mit kompiliert. mit diesen funktionen kann man in ein verzeichnis wechseln oder aus einem verzeichnis ein verzeichnis hoeher wechseln.
	#define MMC_FILE_EXSISTS		FALSE	// TRUE, dann wird die funktion ffileExsists mit kompiliert. mit dieser funktion kann geprueft werden, ob es die datei im aktuellen verzeinis gibt !
	#define MMC_WRITE_STRING		TRUE	// TRUE und MMC_WRITE TRUE, dann wird die funktion ffwrites mit kompiliert. mit dieser funktion koennen strings auf die karte geschrieben werden.

	// vorsicht, da die variable die die sektoren zählt ein short ist (MMC_MAX_CLUSTERS_IN_ROW*fat.secPerClust) !!
	#define MMC_MAX_CLUSTERS_IN_ROW 256 	// gibt an wie viele cluster am stück ohne fat-lookup geschrieben bzw gelesen werden können, wenn die fat nicht fragmentiert ist !

	// wenn "#define INLINE inline" dann werden die statischen funktionen inline kompiliert. macht code groeßer, aber auch schneller!
	//#define INLINE inline
	#define INLINE


	//#####################################################################################
	// SPI EINSTELLUNGEN

	// wenn MAX_SPEED FALSE dann wird die SPI bus geschwindigkeit nicht hoch gesetzt. ist zum testen von hardware die nicht richtig läuft
	#define MMC_MAX_SPEED 		TRUE
	#define MMC_STATUS_INFO 	FALSE

	// software spi? wenn TRUE muessen in der mmc.h noch die pins eingestellt werden über die die spi kommunikation laufen soll
	// es werden nur pins an einem port unterstuetzt !
	#define MMC_SOFT_SPI 	FALSE


	//#####################################################################################
	// TYPDEFINITIONEN

	// pointer typ: fptr, auf eine funktion mit einem arugment: arg und ohne rückgabewert. arg ist vom typ: "unsigned char *" !
	// wird für die ausgabe benutzt, so kann in der main auch einfach eine eigene
	// uart routine verwendet werden! beispiel aufruf von ffls: "ffls(uputs);" wobei uputs eine funktion
	// der form "uputs(unsigned char *s)" ist
	typedef void(*fptr)(unsigned char* arg);


	//#####################################################################################
	// ZEIT FUNKTIONEN

	// rueckgabe wert in folgendem format:
	// bits [0-4]   => sekunde, gueltige werte: 0-29, laut spezifikation wird diese zahl beim anzeigen mit 2 multipliziert, womit man dann auf eine anzeige von 0-58 sekunden kommt !
	// bits [5-10]  => minuten, gueltige werte: 0-59
	// bits [11-15] => stunden, gueltige werte: 0-23
	// macht insgesammt 16 bit also eine 2 byte variable !
	/**extern unsigned short fat_getTime(); DIESE FUNKTION WURDE SCHON IN DER fat.c ERSTELLT UND IN DER fat.h DEKLARIERT !**/

	// rueckgabe wert in folgendem format:
	// bits [0-4]  => tag es monats, gueltige werte: 1-31
	// bits [5-8]  => monat des jahres, gueltige werte: 1-12
	// bits [9-15] => anzahl jahre seit 1980, gueltige werte: 0-127 (moegliche jahre demnach: 1980-2107)
	// macht insgesammt 16 bit also eine 2 byte variable !
	/**extern unsigned short fat_getDate(); DIESE FUNKTION WURDE SCHON IN DER fat.c ERSTELLT UND IN DER fat.h DEKLARIERT !**/

#endif
