
#define SERIAL_PORT 1
#define TEST_LED    1

#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include "cdef.h"
#include "CycleSystem.h"


#include "config.h"  // Hier werden alle nötigen Konfigurationen vorgenommen !
#include "file.h"
#include "fat.h"
#include "mmc.h"    // Hardware abhängig 


#define nRECORD_TIME 60*75  // 75min in [s]

#if SERIAL_PORT
  #include "uart.h"    // Hardware abhängig, es kann auch eine eigene eingebunden werden !
#endif


// Dateinamen muessen in diesem Format sein ! 
// Man beachte die Groesse des Arrays und die Grossbuchstaben!
unsigned char gcfile_name[13]="T0000000___";


uint8  gu8Record_Started = 0;

uint16 u16ADC_Conversions = 0;
uint16 u16ADC_Conversions_per_Sec = 0;



SIGNAL(ADC_vect)
{
  ADCSRA |= (1 << ADSC); //ADC starten

  // Ready
  if (gu8Record_Started)
  {
    // Result
    ffwrite(ADCH);
  }
                  
  u16ADC_Conversions++;
}


uint8 u8ADC_Start_Int(void)
{
  u16ADC_Conversions = 0;
  sbi(ADCSRA, ADIE);  //ADC starten
  ADCSRA |= (1 << ADSC); //ADC starten
  return 0;
}

uint8 u8ADC_Stop_Int(void)
{
  cbi(ADCSRA, ADIE);  //ADC starten
  return 0;
}

uint8 u8Init_ADC_Int(void)
{
  // PC.0  als Trisate Eingang
  DDRC  &= 0x0FE;
  PORTC |= 0x001;
    
  ADCSRA |= 1 << ADEN;  //ADC starten
  
  ADMUX  |= 1 << ADLAR; //ADCH oberer 8 Bit, ADCL untere 2 Bit

  ADCSRA &= 0xF8;

  // Vorteiler auf 64, bei 8 Mhz
  // 125kHz ADc-Frequenz = 8 s
  // -> "Normal Conversion" 8s*13 = 104s
  // Entspricht 9615 Hz
  // ADCSRA |= (1 << ADPS2) + (1 << ADPS1);


  // Vorteiler auf 32, bei 8 Mhz
  // 250kHz ADc-Frequenz = 4 s
  // -> "Normal Conversion" 4s*13 = 52s
  // Entspricht 19330 Hz
  ADCSRA |= (1 << ADPS2) + (1 << ADPS0); 
 
  //Die "First Conversion" machen
  ADCSRA |= (1 << ADIF); //ADIF rckseten "ADIF is cleared by writing 
                         // a logical one to the flag."
  ADCL = 0;
  ADCH = 0; 
  
  
  return 0;
}




//  ------------------ ADC Pol ------------------------ 

void vStart_ADC_Pol(void)
{
  ADCL = 0;
  ADCH = 0; 

  ADCSRA |= (1 << ADIF); //ADIF rckseten "ADIF is cleared by writing 
                         // a logical one to the flag."
  ADCSRA |= (1 << ADSC); //ADC starten
}

uint8 u8ADC_Ready_Pol(void)
{
  if (ADCSRA & (1 << ADIF)) 
  {  
    return 1;
  }	 
  return 0;
}


uint8 u8ADC_Result8_Pol(void)
{
  vStart_ADC_Pol();
	while (u8ADC_Ready_Pol() == 0) {};
  return ADCH;
}


uint8 u8Init_ADC_Pol(void)
{
  uint8  litemp;

  // PC.0  als Trisate Eingang
  DDRC  &= 0x0FE;
  PORTC |= 0x001;
    
  ADCSRA |= 1 << ADEN;  //ADC starten
  
  ADMUX  |= 1 << ADLAR; //ADCH oberer 8 Bit, ADCL untere 2 Bit


  ADCSRA &= 0xF8;

  // Vorteiler auf 64, bei 8 Mhz
  // 125kHz ADc-Frequenz = 8 s
  // -> "Normal Conversion" 8s*13 = 104s
  // Entspricht 9615 Hz
  // ADCSRA |= (1 << ADPS2) + (1 << ADPS1);


  // Vorteiler auf 32, bei 8 Mhz
  // 250kHz ADc-Frequenz = 4 s
  // -> "Normal Conversion" 4s*13 = 52s
  // Entspricht 19330 Hz
  ADCSRA |= (1 << ADPS2) + (1 << ADPS0);

 
  //Die "First Conversion" machen
  litemp = u8ADC_Result8_Pol();
  
  return 0;
}



uint8 u8Init_Record(void)
{  
  #if SERIAL_PORT
    uputs((unsigned char*)"\nBoot");
  #endif

  // Versuch Karte zu Initialisieren, bis es klappt. Umbedingt so, weil die Initialiesierung nicht immer auf anhieb klappt.
  while (FALSE==mmc_init())
  {
    #if SERIAL_PORT
      uputs((unsigned char*)"\nInit");
    #else
      nop();
    #endif 
  }

  #if SERIAL_PORT  
    uputs((unsigned char*)"...");
  #endif

  // Fat initialisieren. Nur wenn das klappt sind weitere Aktionen sinnvoll, sonst aendet das Programm !
  return (fat_loadFatData());
}


uint8 u8Start_Record(void)
{
  uint16 u16FileCounter;
  uint8  u8i;
  char str[8];

  // String zum in die Datei schreiben.

  #if SERIAL_PORT
    uputs((unsigned char*)"Ok\n");    // Wenn auf dem terminal "Boot... OK" zu lesen ist, ist Init OK. Jetzt kann man schreiben/anhaengen/lesen
  #endif

  gu8Record_Started = 0;


  gcfile_name[8]  = 'D';
  gcfile_name[9]  = 'A';
  gcfile_name[10] = 'T';

  u16FileCounter = 0;
  itoa(u16FileCounter, str, 10);

  if (strlen(str) > 0)
  {  
    for (u8i = 0; u8i < strlen(str); u8i++)
    {
      gcfile_name[7 - u8i] = str[strlen(str) - 1 - u8i];
    }
  }

  while (MMC_FILE_NEW != ffopen(gcfile_name))
  {
    ffclose();

    u16FileCounter++;
    itoa(u16FileCounter, str, 10);

    if (strlen(str) > 0)
    {  
      for (u8i = 0; u8i < strlen(str); u8i++)
      {
        gcfile_name[7 - u8i] = str[strlen(str) - 1 - u8i];
      }
    }
  }


  gu8Record_Started = 1;

  #if SERIAL_PORT
    uputs((unsigned char*)gcfile_name);
    uputs((unsigned char*)" opened\n");
  #endif

  u8Init_ADC_Int();
  u8ADC_Start_Int();


  return 0;
}


uint8 u8Stop_Record(void)
{
  // String zum in die Datei schreiben.
  char str[16];

  u8ADC_Stop_Int();

  gu8Record_Started = 0;


  ffclose();

  gcfile_name[8]  = 'T';
  gcfile_name[9]  = 'X';
  gcfile_name[10] = 'T';

  if (MMC_FILE_NEW == ffopen(gcfile_name))
  {
    itoa(u16ADC_Conversions_per_Sec, str, 10);

   // Schreibt String auf Karte ! Nur richtige Strings koennen mit ffwrites geschrieben werden !
   ffwrites((uint8*)str);

   // Neue Zeile in der Datei
   ffwrite(0x0D);
   ffwrite(0x0A);

    ffclose();
  }


  #if SERIAL_PORT
    uputs((unsigned char*)"\nExit");
  #endif

  return 0;
} 



//*****************************************************************************************************************

#if TEST_LED

uint8 Toggle_LED(void)
{
  if (bit_is_set(PORTC,2)) 
	{
	  cbi(PORTC,2);
	}
	else 
	{
	  sbi(PORTC,2);
	}
  return 1;
}


uint8 Init_LED(void)
{
  // PC.2 als Ausgang
  DDRC |= 4;

  return 0;
}

#endif




void Callback_All_100ms_HP(void)
{
  #if TEST_LED
    if (gu8Record_Started)
    {
      Toggle_LED(); 
    }
  #endif
}

void Callback_All_1s_HP(void)
{
  #if TEST_LED
    if (!gu8Record_Started)
    {
      Toggle_LED(); 
    }
  #endif
    
  u16ADC_Conversions_per_Sec = u16ADC_Conversions;   
  u16ADC_Conversions = 0;

} 


void Callback_All_1s_LP(void)
{
  static uint16 u16Timer = 0;


  u16Timer++;

  if (u16Timer > (nRECORD_TIME)) 
  {
    u8Stop_Record();
  }
  

  #if SERIAL_PORT
    if (gu8Record_Started)
    {
      uputs("on ");
    }
    else
    {
      uputs("off ");
    }  
  #endif
} 


//*****************************************************************************************************************
int main(void)
{
  uint8 lui8i, lui8t;


  /* Warteschleife */
  /*for (lui8i = 0; lui8i<=255; lui8i++)
  {
    for (lui8t = 0; lui8t<=255; lui8t++)
    {
      nop();
    }
  }*/

  gu8Record_Started = 0;

  #if SERIAL_PORT
    // Uart initialisierung zu Ausgabe von Daten
    uinit();
  #endif

  #if TEST_LED
    Init_LED();
  #endif


  sei(); 

  
  CycSys_Start(NULL,
               NULL,
               NULL,
               &Callback_All_100ms_HP,
               &Callback_All_1s_HP,

               NULL,
               NULL,
               NULL,
               NULL,
               &Callback_All_1s_LP);

  if(u8Init_Record())
  {
    u8Start_Record();
  }


  while (1)
  {
    CycSys_Idle();
  }
  return 0;
}
