
#define SERIAL_PORT 1
#define TEST_LED    1

#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include "cdef.h"
#include "CycleSystem.h"


#include "config.h"  // Hier werden alle nötigen Konfigurationen vorgenommen !
#include "file.h"
#include "fat.h"
#include "mmc.h"    // Hardware abhängig 

#if SERIAL_PORT
  #include "uart.h"    // Hardware abhängig, es kann auch eine eigene eingebunden werden !
#endif



// Dateinamen muessen in diesem Format sein ! 
// Man beachte die Groesse des Arrays und die Grossbuchstaben!
unsigned char gcfile_name[13]="T0000000___";


uint8  gu8Record_Started = 0;

uint16 u16ADC_Conversions = 0;
uint16 u16ADC_Conversions_per_Sec = 0;



SIGNAL(ADC_vect)
{
  ADCSRA |= (1 << ADSC); //ADC starten

  // Ready
  if (gu8Record_Started)
  {
    // Result
    ffwrite(ADCH);
  }
                  
  u16ADC_Conversions++;
}


uint8 u8ADC_Start_Int(void)
{
  u16ADC_Conversions = 0;
  sbi(ADCSRA, ADIE);  //ADC starten
  ADCSRA |= (1 << ADSC); //ADC starten
  return 0;
}

uint8 u8ADC_Stop_Int(void)
{
  cbi(ADCSRA, ADIE);  //ADC starten
  return 0;
}

uint8 u8Init_ADC_Int(void)
{
  // PC.0  als Trisate Eingang
  DDRC  &= 0x0FE;
  PORTC |= 0x001;
    
  ADCSRA |= 1 << ADEN;  //ADC starten
  
  ADMUX  |= 1 << ADLAR; //ADCH oberer 8 Bit, ADCL untere 2 Bit

  ADCSRA &= 0xF8;

  // Vorteiler auf 64, bei 8 Mhz
  // 125kHz ADc-Frequenz = 8 s
  // -> "Normal Conversion" 8s*13 = 104s
  // Entspricht 9615 Hz
  // ADCSRA |= (1 << ADPS2) + (1 << ADPS1);


  // Vorteiler auf 32, bei 8 Mhz
  // 250kHz ADc-Frequenz = 4 s
  // -> "Normal Conversion" 4s*13 = 52s
  // Entspricht 19330 Hz
  ADCSRA |= (1 << ADPS2) + (1 << ADPS0); 
 
  //Die "First Conversion" machen
  ADCSRA |= (1 << ADIF); //ADIF rckseten "ADIF is cleared by writing 
                         // a logical one to the flag."
  ADCL = 0;
  ADCH = 0; 
  
  
  return 0;
}




//  ------------------ ADC Pol ------------------------ 

void vStart_ADC_Pol(void)
{
  ADCL = 0;
  ADCH = 0; 

  ADCSRA |= (1 << ADIF); //ADIF rckseten "ADIF is cleared by writing 
                         // a logical one to the flag."
  ADCSRA |= (1 << ADSC); //ADC starten
}

uint8 u8ADC_Ready_Pol(void)
{
  if (ADCSRA & (1 << ADIF)) 
  {  
    return 1;
  }	 
  return 0;
}


uint8 u8ADC_Result8_Pol(void)
{
  vStart_ADC_Pol();
	while (u8ADC_Ready_Pol() == 0) {};
  return ADCH;
}


uint8 u8Init_ADC_Pol(void)
{
  uint8  litemp;

  // PC.0  als Trisate Eingang
  DDRC  &= 0x0FE;
  PORTC |= 0x001;
    
  ADCSRA |= 1 << ADEN;  //ADC starten
  
  ADMUX  |= 1 << ADLAR; //ADCH oberer 8 Bit, ADCL untere 2 Bit


  ADCSRA &= 0xF8;

  // Vorteiler auf 64, bei 8 Mhz
  // 125kHz ADc-Frequenz = 8 s
  // -> "Normal Conversion" 8s*13 = 104s
  // Entspricht 9615 Hz
  // ADCSRA |= (1 << ADPS2) + (1 << ADPS1);


  // Vorteiler auf 32, bei 8 Mhz
  // 250kHz ADc-Frequenz = 4 s
  // -> "Normal Conversion" 4s*13 = 52s
  // Entspricht 19330 Hz
  ADCSRA |= (1 << ADPS2) + (1 << ADPS0);

 
  //Die "First Conversion" machen
  litemp = u8ADC_Result8_Pol();
  
  return 0;
}



uint8 u8Init_Record(void)
{  
  #if SERIAL_PORT
    uputs((unsigned char*)"\nBoot");
  #endif

  // Versuch Karte zu Initialisieren, bis es klappt. Umbedingt so, weil die Initialiesierung nicht immer auf anhieb klappt.
  while (FALSE==mmc_init())
  {
    #if SERIAL_PORT
      uputs((unsigned char*)"\nInit");
    #else
      nop();
    #endif 
  }

  #if SERIAL_PORT  
    uputs((unsigned char*)"...");
  #endif

  // Fat initialisieren. Nur wenn das klappt sind weitere Aktionen sinnvoll, sonst aendet das Programm !
  return (fat_loadFatData());
}


uint8 u8Start_Record(void)
{
  uint16 u16FileCounter;
  uint8  u8i;
  char str[8];

  // String zum in die Datei schreiben.

  #if SERIAL_PORT
    uputs((unsigned char*)"Ok\n");    // Wenn auf dem terminal "Boot... OK" zu lesen ist, ist Init OK. Jetzt kann man schreiben/anhaengen/lesen
  #endif

  gu8Record_Started = 0;


  gcfile_name[8]  = 'D';
  gcfile_name[9]  = 'A';
  gcfile_name[10] = 'T';

  u16FileCounter = 0;
  itoa(u16FileCounter, str, 10);

  if (strlen(str) > 0)
  {  
    for (u8i = 0; u8i < strlen(str); u8i++)
    {
      gcfile_name[7 - u8i] = str[strlen(str) - 1 - u8i];
    }
  }

  while (MMC_FILE_NEW != ffopen(gcfile_name))
  {
    ffclose();

    u16FileCounter++;
    itoa(u16FileCounter, str, 10);

    if (strlen(str) > 0)
    {  
      for (u8i = 0; u8i < strlen(str); u8i++)
      {
        gcfile_name[7 - u8i] = str[strlen(str) - 1 - u8i];
      }
    }
  }


  gu8Record_Started = 1;

  #if SERIAL_PORT
    uputs((unsigned char*)gcfile_name);
    uputs((unsigned char*)" opened\n");
  #endif

  u8Init_ADC_Int();
  u8ADC_Start_Int();


  return 0;
}


uint8 u8Stop_Record(void)
{
  // String zum in die Datei schreiben.
  char str[16];

  u8ADC_Stop_Int();

  gu8Record_Started = 0;


  ffclose();

  gcfile_name[8]  = 'T';
  gcfile_name[9]  = 'X';
  gcfile_name[10] = 'T';

  if (MMC_FILE_NEW == ffopen(gcfile_name))
  {
    itoa(u16ADC_Conversions_per_Sec, str, 10);

   // Schreibt String auf Karte ! Nur richtige Strings koennen mit ffwrites geschrieben werden !
   ffwrites((uint8*)str);

   // Neue Zeile in der Datei
   ffwrite(0x0D);
   ffwrite(0x0A);

    ffclose();
  }


  #if SERIAL_PORT
    uputs((unsigned char*)"\nExit");
  #endif

  return 0;
} 



//*****************************************************************************************************************

#if TEST_LED

uint8 Toggle_LED(void)
{
  if (bit_is_set(PORTC,2)) 
	{
	  cbi(PORTC,2);
	}
	else 
	{
	  sbi(PORTC,2);
	}
  return 1;
}


uint8 Init_LED(void)
{
  // PC.2 als Ausgang
  DDRC |= 4;

  return 0;
}

#endif


uint8 Init_Input(void)
{
  // PC.3, PC.4 als Eingang
  cbi(DDRC, 3);
  cbi(DDRC, 4);

  // Pull Up einschalten
  sbi(PORTC, 3);
  sbi(PORTC, 4);
  return 0;
}

#define boGetJumper1() (!(PINC & 8)) // Low aktiv
#define boGetJumper2() (!(PINC & 16)) // Low aktiv 

#define boGetJumper() (boGetJumper1() || boGetJumper2())


void Callback_All_100ms_HP(void)
{
  #if TEST_LED
    if (gu8Record_Started)
    {
      Toggle_LED(); 
    }
  #endif
}

void Callback_All_1s_HP(void)
{
  #if TEST_LED
    if (!gu8Record_Started)
    {
      Toggle_LED(); 
    }
  #endif
    
  u16ADC_Conversions_per_Sec = u16ADC_Conversions;   
  u16ADC_Conversions = 0;

} 


void Callback_All_1s_LP(void)
{
  static uint8 u8LastValue = 0;

  if (boGetJumper() != u8LastValue)
  {
    if (boGetJumper())
    {
      if(u8Init_Record())
      {
        u8Start_Record();
      }    
    }
    else
    {
      u8Stop_Record();
    }
  

    u8LastValue = boGetJumper();
  }

  #if SERIAL_PORT
    /*if (gu8Record_Started)
    {
      uputs("on ");
    }
    else
    {
      uputs("off ");
    } */  

    {
      unsigned char str[8];
      itoa((boGetJumper() << 1) +(boGetJumper1() << 1) + boGetJumper2(), (char*) str, 10);
      uputs(str);
    }
  #endif
} 


//*****************************************************************************************************************
int main(void)
{

  gu8Record_Started = 0;

  #if SERIAL_PORT
    // Uart initialisierung zu Ausgabe von Daten
    uinit();
  #endif

  #if TEST_LED
    Init_LED();
  #endif

  Init_Input();

  sei(); 

  
  CycSys_Start(NULL,
               NULL,
               NULL,
               &Callback_All_100ms_HP,
               &Callback_All_1s_HP,

               NULL,
               NULL,
               NULL,
               NULL,
               &Callback_All_1s_LP);

  while (1)
  {
    uputs((unsigned char*)"\nPress a to continue");
    while (ukeypressed() == 0){CycSys_Idle();}

    switch (ugetc())
    {
      /*case 'a':

        if(u8Init_Record())
        {
          u8Start_Record();
        }

      break;
      case 'b':
        u8Stop_Record();  
      break; */
      
			case 'c':
			  u8Init_ADC_Pol();
        {
          unsigned char str[8];

          while (ukeypressed() == 0)
          { 
          
            uputc(10);uputc(13);
            itoa(u8ADC_Result8_Pol(), (char*) str, 10);
            uputs(str);
          }
				  ugetc();
        }			  
				uputs((unsigned char*)"\nExit");
			break; 

			case 'd':
        gu8Record_Started = 0;
        u8Init_ADC_Int();
        u8ADC_Start_Int();
        {
          unsigned char str[15];

          while (ukeypressed() == 0)
          { 
          
            uputc(13);uputc(10);
            itoa(u16ADC_Conversions_per_Sec, (char*) str, 10);
            uputs(str);
          }
				  ugetc();
        }
        			  
				uputs((unsigned char*)"\nExit");
        u8ADC_Stop_Int();
			break; 

			case 'e':
        gu8Record_Started = 0;
        u8Init_ADC_Int();
        u8ADC_Start_Int();
        {
          unsigned char str[15];

          while (ukeypressed() == 0)
          { 
          
            uputc(13);uputc(10);
            itoa((boGetJumper() << 1) +(boGetJumper1() << 1) + boGetJumper2(), (char*) str, 10);
            uputs(str);
          }
				  ugetc();
        }
        			  
				uputs((unsigned char*)"\nExit");
        u8ADC_Stop_Int();
			break; 

	      default:
	        uputs((unsigned char*)"\n a,b, c or d");
	      break;

    }
  }
  return 0;
}
